"""
Copyright (c) 2023 CEA-List

This program and the accompanying materials are made available under the
terms of the Eclipse Public License 2.0 which is available at
http://www.eclipse.org/legal/epl-2.0.

SPDX-License-Identifier: EPL-2.0
"""

import unittest
import aidge_core

class test_operator_binding(unittest.TestCase):
    """Very basic test to make sure the python APi is not broken.
    Can be remove in later stage of the developpement.
    """
    def setUp(self):
        self.generic_operator = aidge_core.GenericOperator("FakeConv", 1, 1, 1).get_operator()

    def tearDown(self):
        pass

    def test_default_name(self):
        op_type = "Conv"
        gop = aidge_core.GenericOperator(op_type, 1, 1, 1, "FictiveName")
        # check node name is not operator type
        self.assertNotEqual(gop.name(), "Conv")
        # check node name is not default
        self.assertNotEqual(gop.name(), "")

    def test_param_bool(self):
        self.generic_operator.add_parameter("bool", True)
        self.assertEqual(self.generic_operator.get_parameter("bool"), True)

    def test_param_int(self):
        self.generic_operator.add_parameter("int", 1)
        self.assertEqual(self.generic_operator.get_parameter("int"), 1)

    def test_param_float(self):
        self.generic_operator.add_parameter("float", 2.0)
        self.assertEqual(self.generic_operator.get_parameter("float"), 2.0)

    def test_param_str(self):
        self.generic_operator.add_parameter("str", "value")
        self.assertEqual(self.generic_operator.get_parameter("str"), "value")

    def test_param_l_int(self):
        self.generic_operator.add_parameter("l_int", [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15])
        self.assertEqual(self.generic_operator.get_parameter("l_int"), [1,2,3,4,5,6,7,8,9,10,11,12,13,14,15])

    def test_param_l_bool(self):
        self.generic_operator.add_parameter("l_bool", [True, False, False, True])
        self.assertEqual(self.generic_operator.get_parameter("l_bool"), [True, False, False, True])

    def test_param_l_float(self):
        self.generic_operator.add_parameter("l_float", [2.0, 1.0])
        self.assertEqual(self.generic_operator.get_parameter("l_float"), [2.0, 1.0])

    def test_param_l_str(self):
        self.generic_operator.add_parameter("l_str", ["ok"])
        self.assertEqual(self.generic_operator.get_parameter("l_str"), ["ok"])

if __name__ == '__main__':
    unittest.main()