/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#ifndef AIDGE_OPERATORIMPL_H_
#define AIDGE_OPERATORIMPL_H_

#include <cstddef>
#include <vector>
#include "aidge/utils/Types.h"

namespace Aidge {
class OperatorImpl {
public:
    virtual void forward(){};
    virtual void backward(){};

    /**
     * @brief Minimum amount of data from a specific input required by the
     * implementation to be run.
     *
     * @param inputIdx Index of the input analysed.
     * @return std::size_t
     */
    virtual NbElts_t getNbRequiredData(const IOIndex_t inputIdx) const = 0;

    // Amount of input data that cannot be overwritten during the execution.
    virtual NbElts_t getNbRequiredProtected(const IOIndex_t inputIdx) const = 0;

    // Memory required at an output for a given input size.
    virtual NbElts_t getRequiredMemory(const IOIndex_t outputIdx, const std::vector<DimSize_t> &inputsSize) const = 0;

    /**
     * @brief Total amount of consumed data from a specific input.
     *
     * @param inputIdx Index of the input analysed.
     * @return DimSize_t
     */
    virtual NbElts_t getNbConsumedData(const IOIndex_t inputIdx) const = 0;

    /**
     * @brief Total amount of produced data ready to be used on a specific output.
     *
     * @param outputIdx Index of the output analysed.
     * @return DimSize_t
     */
    virtual NbElts_t getNbProducedData(const IOIndex_t outputIdx) const = 0;

    /**
     * @brief Update the Consummer Producer system by simulating the consumption and production of i/o
     *
     */
    virtual void updateConsummerProducer() = 0;

    virtual ~OperatorImpl() = default;
};
} // namespace Aidge

#endif /* AIDGE_OPERATORIMPL_H_ */
