/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include "aidge/graph/GraphView.hpp"
#include "aidge/graphmatching/GRegex.hpp"

namespace py = pybind11;
namespace Aidge {
void init_GRegex(py::module& m){
    py::class_<GRegex, std::shared_ptr<GRegex>>(m, "GRegex", "GRegex class combines a Node Regex and a list of Graph Regex that together describes a graph pattern as a graph regular expression. GRegex find patterns in a given graph that matches the graph regular expression.")
    .def(py::init<const std::map<std::string,NodeRegex*>&, std::vector<std::string>&>(), py::arg("nodesRegex"), py::arg("seqRegexps"), R"mydelimiter(
    Constructor of GRegex

    :param nodesRegex: Describe the conditions an operator has to fulfill.
    :type nodesRegex: Dict[str,:py:class:`aidge_core.NodeRegex`]
    :param seqRegexps: Describe the graph topological pattern. List of Graph Regex as strings.
    :type seqRegexps: List[str]

    )mydelimiter")
    .def("match", &GRegex::match, py::arg("graphToMatch"), R"mydelimiter(
    Launch the graph matching algorithm on a given graph.
    
    :param graphToMatch: The graph to perform the matching algorithm on.
    :type graphToMatch: :py:class:`aidge_core.GraphView`

    :returns: Matched graph patterns.
    :rtype: :py:class:`aidge_core.Match`

    )mydelimiter")
    ;
}
}
