/********************************************************************************
 * Copyright (c) 2023 CEA-List
 *
 * This program and the accompanying materials are made available under the
 * terms of the Eclipse Public License 2.0 which is available at
 * http://www.eclipse.org/legal/epl-2.0.
 *
 * SPDX-License-Identifier: EPL-2.0
 *
 ********************************************************************************/

#include <pybind11/pybind11.h>
#include <pybind11/stl.h>
#include <stdio.h>

#include "aidge/backend/OperatorImpl.hpp"
#include "aidge/operator/GenericOperator.hpp"
#include "aidge/operator/Operator.hpp"
namespace py = pybind11;
namespace Aidge {

void init_GenericOperator(py::module& m) {
    py::class_<GenericOperator_Op, std::shared_ptr<GenericOperator_Op>, Operator>(m, "GenericOperatorOp",
                                                                                  py::multiple_inheritance())
    .def("get_parameter_type", &GenericOperator_Op::getParameterType)
    .def("get_parameters_name", &GenericOperator_Op::getParametersName)
    .def("add_parameter", &GenericOperator_Op::addParameter<bool>)
    .def("add_parameter", &GenericOperator_Op::addParameter<int>)
    .def("add_parameter", &GenericOperator_Op::addParameter<float>)
    .def("add_parameter", &GenericOperator_Op::addParameter<std::string>)
    .def("add_parameter", &GenericOperator_Op::addParameter<std::vector<bool>>)
    .def("add_parameter", &GenericOperator_Op::addParameter<std::vector<int>>)
    .def("add_parameter", &GenericOperator_Op::addParameter<std::vector<float>>)
    .def("add_parameter", &GenericOperator_Op::addParameter<std::vector<std::string>>)
    .def("get_parameter", [](GenericOperator_Op& self, std::string key) -> py::object {
        /*
        This getParameter method returns the good python type without having to have
        prior knowledge of the parameter type.
        */
        py::object res = py::none();
        std::string paramType = self.getParameterType(key);
        if(paramType == typeid(int).name())
            res = py::cast(self.getParameter<int>(key));
        else if(paramType == typeid(float).name())
            res = py::cast(self.getParameter<float>(key));
        else if(paramType == typeid(bool).name())
            res = py::cast(self.getParameter<bool>(key));
        else if(paramType == typeid(std::string).name())
            res = py::cast(self.getParameter<std::string>(key));
        else if(paramType == typeid(std::vector<bool>).name())
            res = py::cast(self.getParameter<std::vector<bool>>(key));
        else if(paramType == typeid(std::vector<int>).name())
            res = py::cast(self.getParameter<std::vector<int>>(key));
        else if(paramType == typeid(std::vector<float>).name())
            res = py::cast(self.getParameter<std::vector<float>>(key));
        else if(paramType == typeid(std::vector<std::string>).name())
            res = py::cast(self.getParameter<std::vector<std::string>>(key));
        else {
            throw py::key_error("Failed to convert parameter type " + key + ", this issue may come from typeid function which gave an unknown key : [" + paramType + "]. Please open an issue asking to add the support for this key.");
        }
        return res;
    });

    m.def("GenericOperator", &GenericOperator, py::arg("type"), py::arg("nbDataIn"), py::arg("nbIn"), py::arg("nbOut"),
          py::arg("name") = "");
}
}  // namespace Aidge
